package dev.flutter.instrumentation_adapter;

import android.annotation.TargetApi;
import android.os.Build.VERSION_CODES;
import android.support.test.rule.ActivityTestRule;
import io.flutter.app.FlutterActivity;
import java.util.concurrent.LinkedBlockingQueue;
import org.junit.runner.Description;
import org.junit.runner.Runner;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunNotifier;

public class FlutterJUnitRunner extends Runner {

  LinkedBlockingQueue<TestResult> queue;

  @TargetApi(VERSION_CODES.N)
  public FlutterJUnitRunner(Class<FlutterActivity> activityClass) {
    super();

    // Launch Flutter activity.
    Class mainClass = activityClass.getSuperclass();
    ActivityTestRule<FlutterActivity> rule = new ActivityTestRule<>(mainClass);
    rule.launchActivity(null);

    queue = InstrumentationAdapterPlugin.getRegisteredInstance().getQueue();
  }

  @Override
  public Description getDescription() {
    return Description.createTestDescription(
        FlutterJUnitRunner.class, "What should be the getDescription here?");
  }

  @Override
  public void run(RunNotifier notifier) {
    while (true) {
      TestResult testResult = null;
      try {
        testResult = queue.take();
      } catch (InterruptedException e) {
        Thread.currentThread().interrupt(); // Restores the interrupted status
        throw new IllegalStateException(e);
      }
      String status = testResult.getStatus();
      if (status.equals("allTestsFinished")) {
        break;
      }
      Description d =
          Description.createTestDescription(FlutterJUnitRunner.class, testResult.getName());
      if (status.equals("testStarted")) {
        notifier.fireTestStarted(d);
      } else if (status.equals("testFailed")) {
        notifier.fireTestFailure(new Failure(d, new Exception(testResult.getError())));
        notifier.fireTestFinished(d);
      } else if (status.equals("testSucceed")) {
        notifier.fireTestFinished(d);
      }
    }
  }
}