package dev.flutter.instrumentation_adapter;

import android.app.Instrumentation;
import android.os.Build.VERSION_CODES;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.LinkedBlockingQueue;

/** InstrumentationAdapterPlugin */
@android.support.annotation.RequiresApi(api = VERSION_CODES.N)
public class InstrumentationAdapterPlugin implements MethodCallHandler {

  private static CompletableFuture<InstrumentationAdapterPlugin> registeredInstance =
      new CompletableFuture<>();

  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "instrumentation_adapter");
    InstrumentationAdapterPlugin plugin = new InstrumentationAdapterPlugin();
    registeredInstance.complete(plugin);
    channel.setMethodCallHandler(plugin);
  }

  public static InstrumentationAdapterPlugin getRegisteredInstance() {
    try {
      return registeredInstance.get();
    } catch (ExecutionException | InterruptedException e) {
      throw new IllegalThreadStateException("Unable to get registered ResultAdapterPlugin");
    }
  }

  private LinkedBlockingQueue<TestResult> queue;

  public InstrumentationAdapterPlugin() {
    queue = new LinkedBlockingQueue<>();
  }

  public LinkedBlockingQueue<TestResult> getQueue() {
    return queue;
  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    try {
      if (call.method.equals("testStarted")) {
        String name = call.argument("name");
        queue.put(new TestResult("testStarted", name));
        result.success(null);
      } else if (call.method.equals("testFailed")) {
        String name = call.argument("name");
        String error = call.argument("error");
        queue.put(new TestResult("testFailed", name, error));
        result.success(null);
      } else if (call.method.equals("testSucceed")) {
        String name = call.argument("name");
        queue.put(new TestResult("testSucceed", name));
        result.success(null);
      } else if (call.method.equals("allTestsFinished")) {
        queue.put(new TestResult("allTestsFinished"));
        result.success(null);
      } else if (call.method.equals("getPlatformVersion")) {
        result.success("Android " + android.os.Build.VERSION.RELEASE);
      } else {
        result.notImplemented();
      }
    } catch (InterruptedException e) {
      Thread.currentThread().interrupt(); // Restores the interrupted status
      result.error("what code?", "what message?", "what details?");
      throw new IllegalStateException(e);
    }
  }


}
