import 'dart:async';

import 'package:flutter_test/flutter_test.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

class InstrumentationAdapter extends LiveTestWidgetsFlutterBinding {
  static const MethodChannel _channel =
      const MethodChannel('instrumentation_adapter');

  static Future<String> get platformVersion async {
    final String version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }


  Map<String, String> _results = <String, String>{};

  InstrumentationAdapter() {
    tearDownAll(() async {
      await _channel.invokeMethod<void>('allTestsFinished');
    });
  }

  @override
  Future<void> runTest(Future<void> testBody(), VoidCallback invariantTester,
      {String description = '', Duration timeout}) async {

    await _channel.invokeMethod<void>('testStarted',
        <String, dynamic>{'name': description});

    void errorReporter(FlutterErrorDetails errorDetails, String description) async {
      _results[description] = errorDetails.toString();
      await _channel.invokeMethod<void>('testFailed',
          <String, dynamic>{
            'name': description,
            'error': _results[description]});
    }
    reportTestException = errorReporter;

    await super.runTest(testBody, invariantTester, description: description);

    if (_results[description] == null) {
      await _channel.invokeMethod<void>('testSucceed',
          <String, dynamic>{'name': description});
    }
  }
}
